#include "testlib.h"
#include <iostream>
#include <cmath>
#include <vector>
#include <time.h>
#include <map>
#include <set>
#include <deque>
#include <cstdio>
#include <cstdlib>
#include <unordered_map>
#include <bitset>
#include <algorithm>
#include <string>
#include <fstream>
#include <assert.h> 
#include <list>
#include <cstring>
#include <queue>
using namespace std;

const int MAX_N = 200000;

void genBamboo(int n){
	cout << n << "\n";
	for (int i = 0; i < n - 1; i++){
		cout << i + 1 << ' ' << i + 2 << '\n';
	}
}

void genStar(int n){
	cout << n << "\n";
	int t = n / 2;
	for (int i = 0; i < n; i++){
		if (i != t){
			cout << i + 1 << ' ' << t + 1 << '\n';
		}
	}
}

void genRand(int n){
	cout << n << "\n";
	for (int i = 1; i < n; i++){
		int t = rnd.next(i);
		cout << i + 1 << ' ' << t + 1 << '\n';
	}
}


const int MAXN = 500 * 1000 + 7;
int perm[MAXN];
void genCentr(int n){
	cout << n << "\n";
	for (int i = 1; i <= n; i++)
	{
		perm[i] = i;
	}
	shuffle(perm + 1, perm + 1 + n);
	vector<pair<int, int> > edges;
	int curLen = 0, needLen = 1;
	for (int i = 2; i <= n; i++)
	{
		if (curLen == 0)
		{
			edges.push_back(make_pair(1, i));
		}
		else
		{
			edges.push_back(make_pair(i - 1, i));
		}
		curLen++;
		if (curLen == needLen)
		{
			curLen = 0;
			needLen += 2;
		}
	}
	for (int i = 0; i < (int)edges.size(); i++)
	{
		edges[i].first = perm[edges[i].first];
		edges[i].second = perm[edges[i].second];
		if (rnd.next(0, 1) == 0)
		{
			swap(edges[i].first, edges[i].second);
		}
	}
	shuffle(edges.begin(), edges.end());
	for (int i = 0; i < (int)edges.size(); i++){
		printf("%d %d\n", edges[i].first, edges[i].second);
	}
}

struct dsu{
	vector<int>p, w;
	dsu(int n)
	{
		p.resize(4 * n + 17, -1);
		w.resize(4 * n + 17, 1);
	}
	int getp(int x)
	{
		if (p[x] == -1) return x;
		p[x] = getp(p[x]);
		return p[x];
	}
	void join(int u, int v)
	{
		u = getp(u);
		v = getp(v);
		assert(u != v);
		if (w[u] < w[v]) swap(u, v);
		p[v] = u;
		w[u] += w[v];
	}
};

void genKruskal(int n){
	cout << n << "\n";
	for (int i = 1; i <= n; i++){
		perm[i] = i;
	}
	shuffle(perm + 1, perm + 1 + n);
	vector<pair<int, int> > edges;
	dsu d(n + 1);
	int edgesCnt = 0;
	while (edgesCnt < n - 1)
	{
		int u = rnd.next(1, n), v = rnd.next(1, n);
		if (d.getp(u) == d.getp(v)) continue;
		edges.push_back(make_pair(u, v));
		d.join(u, v);
		edgesCnt++;
	}
	for (int i = 0; i < (int)edges.size(); i++)
	{
		edges[i].first = perm[edges[i].first];
		edges[i].second = perm[edges[i].second];
		if (rnd.next(0, 1) == 0)
		{
			swap(edges[i].first, edges[i].second);
		}
	}
	shuffle(edges.begin(), edges.end());
	for (int i = 0; i < (int)edges.size(); i++){
		printf("%d %d\n", edges[i].first, edges[i].second);
	}
}

void genStringRand(int n, int m){
	for(int i = 0; i < n; i++){
		int t = rnd.next(m);
		cout << (char)('a' + t);
	}
	cout << "\n";
}

void genStringBadRand(int n, int m){
	for(int i = 0; i < n; i++){
		int t = 0;
		if(rnd.next(2) == 0){
			t = rnd.next(m);
		}
		cout << (char)('a' + t);
	}
	cout << "\n";
}
void genStringBadRandSuper(int n){
	for(int i = 0; i < n; i++){
		int t = 0;
		if(i < 10){
		    t = i;    
		}
		cout << (char)('a' + t);
	}
	cout << "\n";
}

int main(int argc, char **argv){
	registerGen(argc, argv, 1);
	int typeTree = atoi(argv[1]);
	int typeString = atoi(argv[2]);
	int n = atoi(argv[3]);
	int m = atoi(argv[4]);

	if (typeTree == 0){
		genBamboo(n);
	}
	else if (typeTree == 1){
		genStar(n);
	}
	else if (typeTree == 2){
		genRand(n);
	}
	else if (typeTree == 3){
		genCentr(n);
	}
	else if (typeTree == 4){
		genKruskal(n);
	}
	else if (typeTree == 5){
		genKruskal(n);
	}

	if (typeString == 0){
		genStringRand(n, m);
	}
	else if (typeString == 1){
		genStringBadRand(n, m);
	}
	else if (typeString == 2){
		genStringBadRandSuper(n);
	}


	return 0;
}